"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var chai_1 = require("chai");
var operators_1 = require("rxjs/operators");
var _1 = require(".");
var initial = { count: 0, message: '' };
describe('props.observable', function () {
    describe('lifecycle', function () {
        it('constructs with keys from initial Object', function () {
            var obj = _1.props.observable(initial);
            (0, chai_1.expect)(obj.isDisposed).to.eql(false);
            (0, chai_1.expect)(obj.count).to.eql(0);
            (0, chai_1.expect)(obj.message).to.eql('');
        });
        it('constructs with keys from initial array', function () {
            var obj = _1.props.observable(['count', 'message']);
            (0, chai_1.expect)(obj.isDisposed).to.eql(false);
            (0, chai_1.expect)(obj.count).to.eql(undefined);
            (0, chai_1.expect)(obj.message).to.eql(undefined);
        });
        it('constructs with no initial value', function () {
            var obj = _1.props.observable();
            (0, chai_1.expect)(obj.count).to.eql(undefined);
            (0, chai_1.expect)(obj.message).to.eql(undefined);
        });
        it('makes copy of initial object', function () {
            var initial = { count: 123, message: 'foo' };
            var obj = _1.props.observable(initial);
            (0, chai_1.expect)(obj.count).to.eql(123);
            (0, chai_1.expect)(obj.message).to.eql('foo');
            initial.count = 111;
            initial.message = 'mama';
            (0, chai_1.expect)(obj.count).to.eql(123);
            (0, chai_1.expect)(obj.message).to.eql('foo');
        });
        it('dispose', function () {
            var obj = _1.props.observable(initial);
            obj.dispose();
            (0, chai_1.expect)(obj.isDisposed).to.eql(true);
        });
    });
    describe('get', function () {
        it('fires [get/getting] event with "modified" values', function () {
            var events = {
                getting: [],
                get: [],
            };
            var obj = _1.props.observable(initial);
            obj.$.events$
                .pipe((0, operators_1.filter)(function (e) { return e.type === 'PROP/getting'; }), (0, operators_1.map)(function (e) { return e.payload; }))
                .subscribe(function (e) {
                events.getting.push(e);
                switch (e.key) {
                    case 'message':
                        return e.modify('hello');
                    case 'count':
                        return e.modify(123);
                }
            });
            obj.$.events$
                .pipe((0, operators_1.filter)(function (e) { return e.type === 'PROP/get'; }), (0, operators_1.map)(function (e) { return e.payload; }))
                .subscribe(function (e) {
                events.get.push(e);
            });
            (0, chai_1.expect)(obj.message).to.eql('hello');
            (0, chai_1.expect)(obj.count).to.eql(123);
            (0, chai_1.expect)(events.getting.length).to.eql(2);
            (0, chai_1.expect)(events.getting[0].value).to.eql('');
            (0, chai_1.expect)(events.getting[0].isModified).to.eql(true);
            (0, chai_1.expect)(events.getting[1].value).to.eql(0);
            (0, chai_1.expect)(events.getting[1].isModified).to.eql(true);
            (0, chai_1.expect)(events.get.length).to.eql(2);
            (0, chai_1.expect)(events.get[0].key).to.eql('message');
            (0, chai_1.expect)(events.get[0].value).to.eql('hello');
            (0, chai_1.expect)(events.get[1].key).to.eql('count');
            (0, chai_1.expect)(events.get[1].value).to.eql(123);
        });
    });
    describe('set', function () {
        it('fires [set/setting] event and stores value', function () {
            var events = {
                changing: [],
                changed: [],
            };
            var obj = _1.props.observable(initial);
            obj.changing$.subscribe(function (e) { return events.changing.push(e); });
            obj.changed$.subscribe(function (e) { return events.changed.push(e); });
            (0, chai_1.expect)(obj.message).to.eql('');
            (0, chai_1.expect)(obj.count).to.eql(0);
            (0, chai_1.expect)(events.changing.length).to.eql(0);
            (0, chai_1.expect)(events.changed.length).to.eql(0);
            obj.message = 'foo';
            obj.count = 888;
            (0, chai_1.expect)(events.changing.length).to.eql(2);
            (0, chai_1.expect)(events.changing[0].key).to.eql('message');
            (0, chai_1.expect)(events.changing[0].value.from).to.eql('');
            (0, chai_1.expect)(events.changing[0].value.to).to.eql('foo');
            (0, chai_1.expect)(events.changing[0].isCancelled).to.eql(false);
            (0, chai_1.expect)(events.changing[1].key).to.eql('count');
            (0, chai_1.expect)(events.changing[1].value.from).to.eql(0);
            (0, chai_1.expect)(events.changing[1].value.to).to.eql(888);
            (0, chai_1.expect)(events.changing[1].isCancelled).to.eql(false);
            (0, chai_1.expect)(events.changed.length).to.eql(2);
            (0, chai_1.expect)(events.changed[0].key).to.eql('message');
            (0, chai_1.expect)(events.changed[0].value.from).to.eql('');
            (0, chai_1.expect)(events.changed[0].value.to).to.eql('foo');
            (0, chai_1.expect)(events.changed[1].key).to.eql('count');
            (0, chai_1.expect)(events.changed[1].value.from).to.eql(0);
            (0, chai_1.expect)(events.changed[1].value.to).to.eql(888);
            (0, chai_1.expect)(obj.message).to.eql('foo');
            (0, chai_1.expect)(obj.count).to.eql(888);
        });
        it('cancels set operation', function () {
            var events = {
                setting: [],
                set: [],
            };
            var obj = _1.props.observable(initial);
            obj.changing$.subscribe(function (e) {
                events.setting.push(e);
                e.cancel();
            });
            obj.changed$.subscribe(function (e) { return events.set.push(e); });
            obj.message = 'foo';
            (0, chai_1.expect)(events.setting.length).to.eql(1);
            (0, chai_1.expect)(events.set.length).to.eql(0);
            (0, chai_1.expect)(events.setting[0].isCancelled).to.eql(true);
        });
    });
    it('toObject', function () {
        var obj = _1.props.observable(initial);
        (0, chai_1.expect)(obj.toObject()).to.eql(initial);
        (0, chai_1.expect)(obj.toObject()).to.not.equal(initial);
        obj.count = 8;
        obj.message = 'bang';
        (0, chai_1.expect)(obj.toObject()).to.eql({ count: 8, message: 'bang' });
    });
});
