"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var chai_1 = require("chai");
var _1 = require(".");
var rx_pump_1 = require("./rx.pump");
describe('Pump', function () {
    describe('create', function () {
        it('create with id', function () {
            var bus = _1.rx.bus();
            var pump = rx_pump_1.Pump.create(bus);
            (0, chai_1.expect)(pump.id).to.eql("pump:".concat(_1.rx.bus.instance(bus)));
        });
        it('pump: in/out', function () {
            var bus = _1.rx.bus();
            var pump = rx_pump_1.Pump.create(bus);
            var fired = [];
            var event = { type: 'foo', payload: {} };
            pump.in(function (e) { return fired.push(e); });
            pump.out(event);
            (0, chai_1.expect)(fired.length).to.eql(1);
            (0, chai_1.expect)(fired[0]).to.eql(event);
        });
        it('{ dispose$ } option', function () {
            var _a = _1.rx.disposable(), dispose = _a.dispose, dispose$ = _a.dispose$;
            var bus = _1.rx.bus();
            var pump = rx_pump_1.Pump.create(bus, { dispose$: dispose$ });
            var fired = [];
            var event = { type: 'foo', payload: {} };
            dispose();
            pump.in(function (e) { return fired.push(e); });
            pump.out(event);
            (0, chai_1.expect)(fired).to.eql([]);
        });
        it('filter (pump)', function () {
            var allow = false;
            var filter = function (e) { return allow; };
            var bus = _1.rx.bus();
            var pump = rx_pump_1.Pump.create(bus, { filter: filter });
            var fired = [];
            var event = { type: 'foo', payload: {} };
            pump.in(function (e) { return fired.push(e); });
            pump.out(event);
            (0, chai_1.expect)(fired).to.eql([]);
            allow = true;
            pump.out(event);
            (0, chai_1.expect)(fired.length).to.eql(1);
            (0, chai_1.expect)(fired[0]).to.eql(event);
        });
        it('throw: input not an event-bus', function () {
            var test = function (input) {
                var fn = function () { return rx_pump_1.Pump.create(input); };
                (0, chai_1.expect)(fn).to.throw(/Not a valid event-bus/);
            };
            test(undefined);
            test(null);
            test(123);
            test({});
        });
    });
    describe('connect', function () {
        it('fire events (two-way, duplex)', function () {
            var bus1 = _1.rx.bus();
            var bus2 = _1.rx.bus();
            var pump = rx_pump_1.Pump.create(bus1);
            var fired1 = [];
            var fired2 = [];
            var reset = function () {
                fired1 = [];
                fired2 = [];
            };
            bus1.$.subscribe(function (e) { return fired1.push(e); });
            bus2.$.subscribe(function (e) { return fired2.push(e); });
            bus1.fire({ type: 'foo', payload: {} });
            (0, chai_1.expect)(fired1.length).to.eql(1);
            (0, chai_1.expect)(fired2).to.eql([]);
            rx_pump_1.Pump.connect(pump).to(bus2);
            reset();
            bus1.fire({ type: 'foo', payload: { msg: 'from-1' } });
            (0, chai_1.expect)(fired1.length).to.eql(1);
            (0, chai_1.expect)(fired2.length).to.eql(1);
            (0, chai_1.expect)(fired1[0].payload.msg).to.eql('from-1');
            (0, chai_1.expect)(fired2[0].payload.msg).to.eql('from-1');
            reset();
            bus2.fire({ type: 'foo', payload: { msg: 'from-2' } });
            (0, chai_1.expect)(fired1.length).to.eql(1);
            (0, chai_1.expect)(fired2.length).to.eql(1);
            (0, chai_1.expect)(fired1[0].payload.msg).to.eql('from-2');
            (0, chai_1.expect)(fired2[0].payload.msg).to.eql('from-2');
        });
        it('throw: if connected bus is the same as the one inside the pump', function () {
            var bus = _1.rx.bus();
            var pump = rx_pump_1.Pump.create(bus);
            var fn = function () { return rx_pump_1.Pump.connect(pump).to(bus); };
            (0, chai_1.expect)(fn).to.throw(/to a pump containing itself/);
        });
        describe('dispose', function () {
            it('connection.dispose() method', function () {
                var bus1 = _1.rx.bus();
                var bus2 = _1.rx.bus();
                var pump = rx_pump_1.Pump.create(bus2);
                var connection = rx_pump_1.Pump.connect(pump).to(bus1);
                var fired = {
                    bus1: [],
                    bus2: [],
                    reset: function () {
                        fired.bus1 = [];
                        fired.bus2 = [];
                    },
                };
                bus1.$.subscribe(function (e) { return fired.bus1.push(e); });
                bus2.$.subscribe(function (e) { return fired.bus2.push(e); });
                (0, chai_1.expect)(connection.alive).to.eql(true);
                bus1.fire({ type: 'foo', payload: {} });
                bus2.fire({ type: 'foo', payload: {} });
                (0, chai_1.expect)(fired.bus1.length).to.eql(2);
                (0, chai_1.expect)(fired.bus2.length).to.eql(2);
                fired.reset();
                var count = 0;
                connection.dispose$.subscribe(function () { return count++; });
                connection.dispose();
                (0, chai_1.expect)(count).to.eql(1);
                (0, chai_1.expect)(connection.alive).to.eql(false);
                bus1.fire({ type: 'foo', payload: {} });
                (0, chai_1.expect)(fired.bus1.length).to.eql(1, 'bus1');
                (0, chai_1.expect)(fired.bus2.length).to.eql(0, 'bus2');
                fired.reset();
                bus2.fire({ type: 'foo', payload: {} });
                (0, chai_1.expect)(fired.bus1.length).to.eql(0, 'bus1');
                (0, chai_1.expect)(fired.bus2.length).to.eql(1, 'bus2');
            });
            it('connection({ dispose$ }) parameter option', function () {
                var _a = _1.rx.disposable(), dispose = _a.dispose, dispose$ = _a.dispose$;
                var bus1 = _1.rx.bus();
                var bus2 = _1.rx.bus();
                var pump = rx_pump_1.Pump.create(bus2);
                var connection = rx_pump_1.Pump.connect(pump, { dispose$: dispose$ }).to(bus1);
                (0, chai_1.expect)(connection.alive).to.eql(true);
                var fired = {
                    bus1: [],
                    bus2: [],
                    reset: function () {
                        fired.bus1 = [];
                        fired.bus2 = [];
                    },
                };
                bus1.$.subscribe(function (e) { return fired.bus1.push(e); });
                bus2.$.subscribe(function (e) { return fired.bus2.push(e); });
                var count = 0;
                connection.dispose$.subscribe(function () { return count++; });
                dispose();
                (0, chai_1.expect)(count).to.eql(1);
                (0, chai_1.expect)(connection.alive).to.eql(false);
                bus1.fire({ type: 'foo', payload: {} });
                (0, chai_1.expect)(fired.bus1.length).to.eql(1, 'bus1');
                (0, chai_1.expect)(fired.bus2.length).to.eql(0, 'bus2');
                fired.reset();
                bus2.fire({ type: 'foo', payload: {} });
                (0, chai_1.expect)(fired.bus1.length).to.eql(0, 'bus1');
                (0, chai_1.expect)(fired.bus2.length).to.eql(1, 'bus2');
            });
        });
        describe('filter (connection)', function () {
            it('clone and filter the connection', function () {
                var bus1 = _1.rx.bus();
                var bus2 = _1.rx.bus();
                var pump = rx_pump_1.Pump.create(bus2);
                var conn1 = rx_pump_1.Pump.connect(pump).to(bus1);
                var conn2 = conn1.filter(function () { return true; });
                (0, chai_1.expect)(conn1).to.not.equal(conn2);
            });
            it('dispose child (filter/cloned) connection does not dispose the parent connection', function () {
                var bus1 = _1.rx.bus();
                var bus2 = _1.rx.bus();
                var pump = rx_pump_1.Pump.create(bus2);
                var conn1 = rx_pump_1.Pump.connect(pump).to(bus1);
                var conn2 = conn1.filter(function () { return true; });
                (0, chai_1.expect)(conn1.alive).to.eql(true);
                (0, chai_1.expect)(conn2.alive).to.eql(true);
                conn2.dispose();
                (0, chai_1.expect)(conn1.alive).to.eql(true);
                (0, chai_1.expect)(conn2.alive).to.eql(false);
            });
            it('dispose root/parent connection disposes child filter/cloned connections', function () {
                var bus1 = _1.rx.bus();
                var bus2 = _1.rx.bus();
                var pump = rx_pump_1.Pump.create(bus2);
                var conn1 = rx_pump_1.Pump.connect(pump).to(bus1);
                var conn2 = conn1.filter(function () { return true; });
                var conn3 = conn1.filter(function () { return false; });
                (0, chai_1.expect)(conn1.alive).to.eql(true);
                (0, chai_1.expect)(conn2.alive).to.eql(true);
                (0, chai_1.expect)(conn3.alive).to.eql(true);
                conn1.dispose();
                (0, chai_1.expect)(conn1.alive).to.eql(false);
                (0, chai_1.expect)(conn2.alive).to.eql(false);
                (0, chai_1.expect)(conn3.alive).to.eql(false);
            });
            it('filter events via { filter } input parameter', function () {
                var bus1 = _1.rx.bus();
                var bus2 = _1.rx.bus();
                var pump = rx_pump_1.Pump.create(bus2);
                var filter = function (e) { return e.event.payload.msg !== 'filter-me'; };
                rx_pump_1.Pump.connect(pump, { filter: filter }).to(bus1);
                var fired = {
                    bus1: [],
                    bus2: [],
                    reset: function () {
                        fired.bus1 = [];
                        fired.bus2 = [];
                    },
                };
                bus1.$.subscribe(function (e) { return fired.bus1.push(e); });
                bus2.$.subscribe(function (e) { return fired.bus2.push(e); });
                bus1.fire({ type: 'foo', payload: {} });
                bus2.fire({ type: 'foo', payload: {} });
                (0, chai_1.expect)(fired.bus1.length).to.eql(2);
                (0, chai_1.expect)(fired.bus2.length).to.eql(2);
                fired.reset();
                bus1.fire({ type: 'foo', payload: { msg: 'filter-me' } });
                (0, chai_1.expect)(fired.bus1.length).to.eql(1);
                (0, chai_1.expect)(fired.bus2.length).to.eql(0);
                fired.reset();
                bus2.fire({ type: 'foo', payload: { msg: 'filter-me' } });
                (0, chai_1.expect)(fired.bus1.length).to.eql(0);
                (0, chai_1.expect)(fired.bus2.length).to.eql(1);
                fired.reset();
            });
            it('filter events via [connection.filter(fn)] clone', function () {
                var bus1 = _1.rx.bus();
                var bus2 = _1.rx.bus();
                var pump = rx_pump_1.Pump.create(bus2);
                rx_pump_1.Pump.connect(pump)
                    .filter(function (e) { return e.event.payload.msg !== 'filter-me'; })
                    .to(bus1);
                var fired = {
                    bus1: [],
                    bus2: [],
                    reset: function () {
                        fired.bus1 = [];
                        fired.bus2 = [];
                    },
                };
                bus1.$.subscribe(function (e) { return fired.bus1.push(e); });
                bus2.$.subscribe(function (e) { return fired.bus2.push(e); });
                bus1.fire({ type: 'foo', payload: {} });
                bus2.fire({ type: 'foo', payload: {} });
                (0, chai_1.expect)(fired.bus1.length).to.eql(2);
                (0, chai_1.expect)(fired.bus2.length).to.eql(2);
                fired.reset();
                bus1.fire({ type: 'foo', payload: { msg: 'filter-me' } });
                (0, chai_1.expect)(fired.bus1.length).to.eql(1);
                (0, chai_1.expect)(fired.bus2.length).to.eql(0);
                fired.reset();
                bus2.fire({ type: 'foo', payload: { msg: 'filter-me' } });
                (0, chai_1.expect)(fired.bus1.length).to.eql(0);
                (0, chai_1.expect)(fired.bus2.length).to.eql(1);
                fired.reset();
            });
        });
    });
});
